data {
  
  int<lower=1> T;                 // # of settlement types
  int<lower=1> R;                 // # of age-sex regions
  int<lower=1> G;                 // # of household geographies
  int<lower=1> U;                 // # of population units
  int<lower=1> K;                 // # of age-sex groups
  int<lower=1> Gi[U];             // max geo index per pop unit
  int<lower=1> GI;                // max geo index
  int<lower=0> g_gi[U,GI];        // key to link geo_idx to geo
  int<lower=0> nhouse[T,G];       // # of households
  int<lower=0> nsingle[T,G];      // # of single-person households
  int<lower=0> nmulti[T,G];       // # of multi-person households 
  int<lower=1> n;                 // total multi-person households
  int<lower=0> H;                 // most multi-person households in a geo
  int<lower=0> imulti[T,G,H];     // indexes of multi-person household
  int<lower=1> N[n];              // additional people per multi-person housing unit
  int<lower=0> M[T,R,K];          // total people sampled per age-sex group
  vector<lower=0>[U] Btot;        // building totals
  vector<lower=0>[U] Ntot;        // approximate population totals
  matrix<lower=0>[T,GI] Bwgt[U];  // building weights
  real<lower=0> nat_tot;          // national population total
}

parameters {
  
  // admin totals
  vector<lower=0>[U] sd_Ntot;
  
  // housing units per building
  row_vector<lower=0>[U] hpb[T];            
  vector[T] mean_hpb;
  real<lower=0> sd_hpb;

  // probability of single-person households
  row_vector<lower=0, upper=1>[G] alpha[T]; 
  vector<lower=0, upper=1>[T] mean_alpha;
  real<lower=0> sd_alpha;
  
  // pph for multi-person households
  vector<lower=0>[n] lambda_hh;
  vector<lower=0>[G] lambda[T];
  
  vector[G] mean_lambda[T];
  vector[T] mean_mean_lambda;
  real<lower=0> sd_mean_lambda;

  vector<lower=0>[G] sd_lambda[T];
  vector<lower=0>[T] mean_sd_lambda;
  real<lower=0> sd_sd_lambda;

  // agesex
  simplex[K] theta[T, R];
}

transformed parameters {
  
  matrix<lower=0>[T,GI] ppb[U]; 
  vector<lower=0>[U] Ntot_hat;

  // people per building
  for(u in 1:U){
    for(t in 1:T){
      for(gi in 1:Gi[u]){
        ppb[u,t,gi] = hpb[t,u] * (1 + (1-alpha[t,g_gi[u,gi]]) * lambda[t,g_gi[u,gi]]);
      }
      for(gi in (Gi[u]+1):GI){
        ppb[u,t,gi] = 0;
      }
    }
    Ntot_hat[u] = Btot[u] * sum(ppb[u] .* Bwgt[u]);
  }
}

model {
  
  // national total
  nat_tot ~ lognormal(log(sum(Ntot_hat)), 0.15/2);
  
  // admin totals
  Ntot ~ lognormal(log(Ntot_hat), sd_Ntot); 
  
  // settlement types (t)
  for(t in 1:T){
    
    // housing units per building (log scale)
    hpb[t] ~ lognormal(mean_hpb[t], sd_hpb);
    
    // people per age-sex group
    for(r in 1:R){
      M[t,r] ~ multinomial(theta[t,r]);
      theta[t,r] ~ dirichlet(rep_vector(1,K)); 
    }
    
    // single person households
    nsingle[t,] ~ binomial(nhouse[t,], alpha[t]);
    alpha[t] ~ beta(mean_alpha[t]/sd_alpha, (1-mean_alpha[t])/sd_alpha);
    
    // additional people in multi-person households (observed values)
    for(g in 1:G){
      
      N[imulti[t,g,1:nmulti[t,g]]] ~ poisson(lambda_hh[imulti[t,g,1:nmulti[t,g]]]);
      
      lambda_hh[imulti[t,g,1:nmulti[t,g]]] ~ lognormal(mean_lambda[t,g], sd_lambda[t,g]);
      
      // adjust likelihood to account for truncated poisson T(1,)
      target += -log1m_exp(-lambda_hh[imulti[t,g,1:nmulti[t,g]]]);
    }
    
    lambda[t] ~ lognormal(mean_lambda[t], sd_lambda[t]);
    
    mean_lambda[t] ~ normal(mean_mean_lambda[t], sd_mean_lambda); 
    sd_lambda[t] ~ normal(mean_sd_lambda[t], sd_sd_lambda);
  }
  
  // priors
  sd_Ntot ~ lognormal(log(0.30/2), 0.5);
  
  mean_mean_lambda ~ normal(0, 5);
  sd_mean_lambda ~ uniform(0, 5);
  
  mean_sd_lambda ~ uniform(0, 5);
  sd_sd_lambda ~ uniform(0, 5);

  mean_alpha ~ uniform(0, 1);
  sd_alpha ~ uniform(0, 1);
  
  mean_hpb ~ normal(0, 5);
  sd_hpb ~ uniform(0, 5);
} 


